#-----------------------------------------------------------------------------
# Rescaler
# by Ockham
# occam24@aol.com

# Custom. Designed to rescale the item to Poser dims without messing up
# my carefully-made groups and mats.
# Begins by finding max and min.
# Fill in the desired RealMaxX in feet or inches, and the part to measure..
# After finding max and min, script rescales
# all dims by the ratio  RealMaxXInPU / ExistingMaxX.
# It also moves Y down to ground at the same time,
# which means we need to rescale the MinY beforehand.
# NewY = OldY * RealMaxXInPU / ExistingMaxX - RescaledMinY
# It also centers the whole thing, which means we need
# to have the Xmove and Zmove dims figured beforehand also.

#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser
import os
scene = poser.Scene()
root = Tk()


ver = float(poser.Version())
if ver < 6.0:
	FeetPerPu=8.0
else:
	FeetPerPu=8.6

#-----------------------------------------------------------------------------
BasePathUncut = poser.AppLocation()
BasePath = BasePathUncut[:(-len("poser.exe"))]
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Rescaler")

		# Vars for Checkbuttons
		self.CenterVar = IntVar()
		self.CenterVar.set(1)
		self.FlooredVar = IntVar()
		self.FlooredVar.set(1)

		self.StatusEntry = Entry(self.master,width=40)
		self.StatusEntry.grid(row=0,column=0,columnspan=2)
		self.StatusEntry.insert(0,"Enter a number followed by x or y or z")

		Label(self.master,text="Foot").grid(row=1,column=0)
		self.FootEntry = Entry(self.master,width=8)
		self.FootEntry.grid(row=1,column=1)

		Label(self.master,text="Inch").grid(row=2,column=0)
		self.InchEntry = Entry(self.master,width=8)
		self.InchEntry.grid(row=2,column=1)

		Label(self.master,text="Centimeter").grid(row=3,column=0)
		self.CMEntry = Entry(self.master,width=8)
		self.CMEntry.grid(row=3,column=1)

		Label(self.master,text="Scale percent").grid(row=4,column=0)
		self.ScalEntry = Entry(self.master,width=8)
		self.ScalEntry.grid(row=4,column=1)

		Label(self.master,text="Floored").grid(row=5,column=0)
		self.BottomCheck = Checkbutton(self.master,variable=self.FlooredVar)
		self.BottomCheck.grid(row=5,column=1)

		Label(self.master,text="Centered").grid(row=6,column=0)
		self.CenterCheck = Checkbutton(self.master,variable=self.CenterVar)
		self.CenterCheck.grid(row=6,column=1)


		# Go/nogo buttons.
		self.buttonGo = Button(self.master, text="Find OBJ file", command=self.HandleGo)
		self.buttonGo.grid(row=7, column=0)

		self.buttonCancel = Button(self.master, text="Close", command=self.HandleCancel)
		self.buttonCancel.grid(row=7, column=1)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -

	def Update(self):
		if scene: scene.ProcessSomeEvents()
		root.lift()
		root.after(100, self.Update)

	# - - - - - - - - - - - - - - - - - -

	def HandleName(self):
		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(os.path.join(BasePath,'runtime','*.OBJ'))
		fn=tkFileDialog.askopenfilename(initialfile=MyPath, defaultextension=[("OBJ file", "*.OBJ *.obj")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		return os.path.abspath(fn)

	# - - - - - - - - - - - - - - - - - -

	def Rescale(self,Name):
		# check both feet and inches.  Whichever one is given, use.

		Feet=0.0
		Scal=0.0
		Use = 'X'

		s=self.FootEntry.get()
		if s:
			parts=string.split(s)
			if len(parts) == 2:
				Use = string.upper(parts[1])
			Feet = float(parts[0])

		s=self.InchEntry.get()
		if s:
			parts=string.split(s)
			if len(parts) == 2:
				Use = string.upper(parts[1])
			Feet = float(parts[0]) / 12.0

		s=self.CMEntry.get()
		if s:
			parts=string.split(s)
			if len(parts) == 2:
				Use = string.upper(parts[1])
			Feet = float(parts[0]) / 30.48 # number of cm in foot

		s=self.ScalEntry.get()
		if s:
			parts=string.split(s)
			if len(parts) == 2:
				Use = string.upper(parts[1])
			elif len(parts) == 1:
				Use = 'X' # really irrelevant, but being consistent
			Scal = float(parts[0]) * 0.01

		NewWidthPU = Feet / FeetPerPu

		XYZ=[]
		Floored=self.FlooredVar.get()
		Centered=self.CenterVar.get()

		# First read the file to get the max's.

		Deadband=0.0005

		Base,ext=os.path.splitext(Name)
		OutName=Base + '_RESCALE' + ext
		fpin=open(Name,"rt")
		fpout=open(OutName,"wt")


		maxX=-10000.0
		minX= 10000.0
		maxY=-10000.0
		minY= 10000.0
		maxZ=-10000.0
		minZ= 10000.0
		InPart=0
		while 1:
		   st=fpin.readline()
		   if not st: break
		   parts = string.split(string.strip(st))

		   if (len(parts)==4) and (parts[0] == 'v'):
			   x = float(parts[1])
			   y = float(parts[2])
			   z = float(parts[3])
			   XYZ.append( (x,y,z) )

			   # get the max and min of all
			   if	x < minX: minX=x
			   elif x > maxX: maxX=x
			   if	y < minY: minY=y
			   elif y > maxY: maxY=y
			   if	z < minZ: minZ=z
			   elif z > maxZ: maxZ=z

		fpin.close()

		# Pick the desired dimension from the Use letter:
		if	 Use == 'Y':
			OldWidth=abs(maxY-minY)
		elif Use == 'Z':
			OldWidth=abs(maxZ-minZ)
		else: # default x
			OldWidth=abs(maxX-minX)

		if Scal: p=Scal
		else: p=NewWidthPU/OldWidth
		if p == 0.0: p = 1.0 # If no entries, use 1:1 scale.

		# Figure these for both floored and centered first.
		Ymove=minY*p
		cX=(maxX+minX)/2.0
		cZ=(maxZ+minZ)/2.0
		Xmove=cX*p
		Zmove=cZ*p

		if not Floored:
			Ymove=0.0

		if not Centered:
			Xmove=0.0
			Zmove=0.0


		# Next re-open the infile and do the rescaling.
		fpin=open(Name,"rt")
		while 1:
		   st=fpin.readline()
		   if not st: break
		   parts = string.split(string.strip(st))

		   if (len(parts)==4) and (parts[0] == 'v'):
			   x = float(parts[1])
			   y = float(parts[2])
			   z = float(parts[3])
			   newX=x*p-Xmove
			   newY=y*p-Ymove
			   newZ=z*p-Zmove
			   fpout.write('v %.5f %.5f %.5f\n' % (newX,newY,newZ))
		   else: # other stuff, just rewrite.
			   fpout.write(st)

		fpin.close()
		fpout.close()
		self.ShowStatus("done, actual scale %f" % p)

	# - - - - - - - - - - - - - - - - - -

	def HandleGo(self):
		# Action here
		Name = self.HandleName()
		self.Rescale(Name)

	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
